==> theia-0.1.1-py3-none-any.whl: theia/aws/network/network.py <==
import pulumi

from pulumi_aws import ec2
from pulumi.output import Output

from collections import defaultdict
from pydantic.dataclasses import dataclass, ConfigDict
from typing import Optional, List, Mapping, Literal, Dict

from theia import ResourceOptions
from theia.utils.logger import Logger
from theia.aws.network.ngw import NatGateway
from theia.aws.network.igw import InternetGateway
from theia.aws.network.vpc import Vpc, VpcSettings
from theia.aws.network.subnet import Subnet, SubnetSettings
from theia.aws.security.sg import SecurityGroup, SecurityGroupSettings
from theia.utils.errors import TheiaNetworkError, TheiaConfigurationError
from theia.aws.network.routes import Route, RouteTable, RouteTableAssociations

logger = Logger("network")


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class NetworkSubnet:
    cidr_block: str | Output[str]
    type: Literal["public", "private"] = "public"
    assign_ipv6: Optional[bool] = False
    map_public_ip: Optional[bool] = False
    availability_zone: Optional[str] = None


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class NetworkOptions:
    create_internet_gateway: Optional[bool] = True
    create_nat_gateway: Literal["none", "single", "per az"] = "single"
    create_vpc_security_group: Optional[bool] = True

    igw_id: Optional[str | Output[str]] = None

    # VPC Config
    secondary_cidr: Optional[str | Output[str]] = None
    enable_dns_hostnames: Optional[bool] = False
    enable_dns_support: Optional[bool] = True

    def __post_init__(self):
        if not self.create_internet_gateway and not self.igw_id:
            raise TheiaNetworkError(
                "Internet Gateway id should be provided in settings if create is set to false"
            )
        if (
            (self.create_nat_gateway != "none")
            and (self.create_nat_gateway != "single")
            and (self.create_nat_gateway != "per az")
        ):
            raise TheiaConfigurationError(
                "create_nat_gateway should be only one of 'none' | 'single' | 'per az'"
            )


class Network(pulumi.ComponentResource):
    def __init__(
        self,
        resource_name: str,
        vpc_cidr: str,
        subnet_specs: List[NetworkSubnet],
        tenancy: str = "default",
        tags: Optional[Mapping[str, str]] = None,
        create_opts: Optional[NetworkOptions] = NetworkOptions(),
        opts: Optional[ResourceOptions] = None,
    ) -> None:
        super().__init__("theia:aws:network", resource_name, None, opts)
        # VPC
        logger.info("Creating VPC...")
        self.__vpc(f"{resource_name}-vpc", vpc_cidr, tenancy, tags, create_opts)

        # Security Group
        if create_opts.create_vpc_security_group:
            self.__sg(resource_name, tags)

        # IGW
        logger.info("Creating Internet Gateway...")
        if create_opts.create_internet_gateway:
            self.__igw(f"{resource_name}-igw", tags)
            igw_id = self.igw.id
        else:
            igw_id = create_opts.igw_id

        # Subnet
        logger.info("Creating Subnets...")
        self.__subnets(resource_name, subnet_specs, tags)

        # NGW
        ngw_per_az = None
        if create_opts.create_nat_gateway != "none":
            logger.info(f"Creating NAT Gateway - {create_opts.create_nat_gateway}...")
            ngw_per_az = self.__ngw(resource_name, create_opts.create_nat_gateway, tags)

        private_subnet_ngw = defaultdict(list)
        if ngw_per_az:
            for i in (self.subnets_per_zone["private"], ngw_per_az):
                for key, value in i.items():
                    private_subnet_ngw[key].append(value)

        # Routes
        logger.info("Creating Route Tables...")
        self.__route(
            resource_name,
            igw_id,
            create_opts.create_nat_gateway,
            private_subnet_ngw,
            tags,
        )

    def __vpc(
        self,
        name: str,
        cidr: str,
        tenancy: str,
        tags: Mapping[str, str] | None,
        create_opts: NetworkOptions,
    ) -> None:
        vpc_settings = VpcSettings(
            secondary_cidr=create_opts.secondary_cidr,
            enable_dns_hostnames=create_opts.enable_dns_hostnames,
            enable_dns_support=create_opts.enable_dns_support,
        )
        self.vpc = Vpc(
            resource_name=name,
            ipv4_cidr=cidr,
            tenancy=tenancy,
            tags=tags,
            settings=vpc_settings,
            opts=ResourceOptions(parent=self),
        )

    def __sg(
        self,
        name: str,
        tags: Mapping[str, str] | None,
    ) -> None:
        sg_settings = SecurityGroupSettings(
            sg_type="ingress",
            traffic_type="All",
            cidr_blocks=[self.vpc.cidr_block],
            add_self=True,
            description=f"Allow inbound from resource in the same vpc: {name}-vpc",
        )
        self.sg = SecurityGroup(
            resource_name=f"{name}-sg",
            vpc_id=self.vpc.id,
            settings=[sg_settings],
            tags=tags,
            opts=ResourceOptions(parent=self, depends_on=[self.vpc]),
        )

    def __igw(self, name: str, tags: Mapping[str, str] | None):
        self.igw = InternetGateway(
            name,
            self.vpc.id,
            tags,
            opts=ResourceOptions(parent=self, depends_on=[self.vpc]),
        )

    def __subnets(
        self,
        name: str,
        subnet_specs: List[NetworkSubnet],
        tags: Mapping[str, str] | None,
    ) -> None:
        self.subnets = {"private": [], "public": []}

        # ! Dirty code
        self.subnets_per_zone = {"private": {}, "public": {}}

        for i, subnet in enumerate(subnet_specs):
            settings = SubnetSettings(
                cidr_block=subnet.cidr_block,
                assign_ipv6_address_on_creation=subnet.assign_ipv6,
                availability_zone=subnet.availability_zone,
                map_public_ip_on_launch=subnet.map_public_ip,
            )

            subnet_name = f"{name}-{i}-{subnet.type}-subnet"
            _subnet = Subnet(
                subnet_name,
                self.vpc.id,
                tags,
                settings,
                opts=ResourceOptions(parent=self, depends_on=[self.vpc]),
            )
            self.subnets[subnet.type].append(_subnet)
            self.subnets_per_zone[subnet.type][subnet.availability_zone] = _subnet

    def __ngw(
        self, name: str, create_opt: str, tags: Mapping[str, str]
    ) -> Optional[Dict[str, ec2.NatGateway]]:
        self.ngws = []
        if create_opt == "per az":
            ngw_per_az = {}
            for az, subnet in self.subnets_per_zone["public"].items():
                ngw_name = f"{name}-{az}-ngw"
                ngw = NatGateway(
                    ngw_name,
                    subnet.id,
                    tags,
                    opts=ResourceOptions(parent=self, depends_on=[subnet]),
                )
                self.ngws.append(ngw)
                ngw_per_az[az] = ngw
            return ngw_per_az
        else:
            name = f"{name}-ngw"
            for az, subnet in self.subnets_per_zone["public"].items():
                subnet = subnet
                break
            self.ngws.append(
                NatGateway(
                    name,
                    subnet.id,
                    tags,
                    opts=ResourceOptions(parent=self, depends_on=[subnet]),
                )
            )
            return None

    def __route(
        self,
        name: str,
        igw_id: str,
        ngw_opts: str,
        private_subnets_ngw: Dict[str, List[ec2.Subnet | ec2.NatGateway]],
        tags: Mapping[str, str],
    ) -> None:
        self.route_tables = []

        # Public Route Table
        route_associations = []
        for i, subnet in enumerate(self.subnets["public"]):
            route_associations.append(RouteTableAssociations(f"public-{i}", subnet.id))
        name = f"{name}-route_table"
        route = Route("0.0.0.0/0", gateway_id=igw_id)

        depends_on = [self.vpc] + self.subnets["public"]
        self.route_tables.append(
            RouteTable(
                f"{name}-public",
                self.vpc.id,
                [route],
                route_associations,
                tags,
                ResourceOptions(parent=self, depends_on=depends_on),
            )
        )

        # Private Route Table
        if private_subnets_ngw and ngw_opts == "per az":
            for az, network in private_subnets_ngw.items():
                route_association = RouteTableAssociations(
                    f"private-{az}", network[0].id
                )

                route = Route("0.0.0.0/0", nat_gateway_id=network[1].id)
                self.route_tables.append(
                    RouteTable(
                        f"{name}-private-{az}",
                        self.vpc.id,
                        [route],
                        [route_association],
                        tags,
                        ResourceOptions(
                            parent=self, depends_on=[self.vpc, network[0], network[1]]
                        ),
                    )
                )
        elif ngw_opts == "single":
            for i, subnet in enumerate(self.subnets["private"]):
                route_association = RouteTableAssociations(f"private-{i}", subnet.id)

                route = Route("0.0.0.0/0", nat_gateway_id=self.ngws[0].id)

                self.route_tables.append(
                    RouteTable(
                        f"{name}-private-{i}",
                        self.vpc.id,
                        [route],
                        [route_association],
                        tags,
                        ResourceOptions(
                            parent=self, depends_on=[self.vpc, subnet, self.ngws[0]]
                        ),
                    )
                )

==> theia-0.1.8.1-py3-none-any.whl: theia/aws/network/network.py <==
import pulumi

from pulumi_aws import ec2
from pulumi.output import Output

from collections import defaultdict
from pydantic.dataclasses import dataclass, ConfigDict
from typing import Optional, List, Mapping, Literal, Dict

from theia import ResourceOptions
from theia.aws.network.ngw import NatGateway
from theia.aws.network.igw import InternetGateway
from theia.aws.network.vpc import Vpc, VpcSettings
from theia.aws.network.subnet import Subnet, SubnetSettings
from theia.aws.security.sg import SecurityGroup, SecurityGroupSettings
from theia.utils.errors import TheiaNetworkError, TheiaConfigurationError
from theia.aws.network.routes import Route, RouteTable, RouteTableAssociations


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class NetworkSubnet:
    cidr_block: str | Output[str]
    type: Literal["public", "private"] = "public"
    assign_ipv6: Optional[bool] = False
    map_public_ip: Optional[bool] = False
    availability_zone: Optional[str] = None


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class NetworkOptions:
    create_internet_gateway: Optional[bool] = True
    create_nat_gateway: Literal["none", "single", "per az"] = "single"
    create_vpc_security_group: Optional[bool] = False

    igw_id: Optional[str | Output[str]] = None

    # VPC Config
    secondary_cidr: Optional[str | Output[str]] = None
    enable_dns_hostnames: Optional[bool] = False
    enable_dns_support: Optional[bool] = True

    def __post_init__(self):
        if not self.create_internet_gateway and not self.igw_id:
            raise TheiaNetworkError(
                "Internet Gateway id should be provided in settings if create is set to false"
            )
        if (
            (self.create_nat_gateway != "none")
            and (self.create_nat_gateway != "single")
            and (self.create_nat_gateway != "per az")
        ):
            raise TheiaConfigurationError(
                "create_nat_gateway should be only one of 'none' | 'single' | 'per az'"
            )


class Network(pulumi.ComponentResource):
    def __init__(
        self,
        resource_name: str,
        vpc_cidr: str,
        subnet_specs: List[NetworkSubnet],
        tenancy: str = "default",
        tags: Optional[Mapping[str, str]] = None,
        create_opts: Optional[NetworkOptions] = NetworkOptions(),
        opts: Optional[ResourceOptions] = None,
    ) -> None:
        super().__init__("theia:aws:network", resource_name, None, opts)
        # VPC
        self.__vpc(f"{resource_name}-vpc", vpc_cidr, tenancy, tags, create_opts)

        # Security Group
        if create_opts.create_vpc_security_group:
            self.__sg(resource_name, tags)

        # IGW
        if create_opts.create_internet_gateway:
            self.__igw(f"{resource_name}-igw", tags)
            igw_id = self.igw.id
        else:
            igw_id = create_opts.igw_id

        # Subnet
        self.__subnets(resource_name, subnet_specs, tags)

        # NGW
        ngw_per_az = None
        if create_opts.create_nat_gateway != "none":
            ngw_per_az = self.__ngw(resource_name, create_opts.create_nat_gateway, tags)

        private_subnet_ngw = defaultdict(list)
        if ngw_per_az:
            for i in (self.subnets_per_zone["private"], ngw_per_az):
                for key, value in i.items():
                    private_subnet_ngw[key].append(value)

        # Routes
        self.__route(
            resource_name,
            igw_id,
            create_opts.create_nat_gateway,
            private_subnet_ngw,
            tags,
        )

    def __vpc(
        self,
        name: str,
        cidr: str,
        tenancy: str,
        tags: Mapping[str, str] | None,
        create_opts: NetworkOptions,
    ) -> None:
        vpc_settings = VpcSettings(
            secondary_cidr=create_opts.secondary_cidr,
            enable_dns_hostnames=create_opts.enable_dns_hostnames,
            enable_dns_support=create_opts.enable_dns_support,
        )
        self.vpc = Vpc(
            resource_name=name,
            ipv4_cidr=cidr,
            tenancy=tenancy,
            tags=tags,
            settings=vpc_settings,
            opts=ResourceOptions(parent=self),
        )

    def __sg(
        self,
        name: str,
        tags: Mapping[str, str] | None,
    ) -> None:
        sg_settings = SecurityGroupSettings(
            sg_type="ingress",
            traffic_type="All",
            cidr_blocks=[self.vpc.cidr_block],
            add_self=True,
            description=f"Allow inbound from resource in the same vpc: {name}-vpc",
        )
        self.sg = SecurityGroup(
            resource_name=f"{name}-sg",
            vpc_id=self.vpc.id,
            settings=[sg_settings],
            tags=tags,
            opts=ResourceOptions(parent=self, depends_on=[self.vpc]),
        )

    def __igw(self, name: str, tags: Mapping[str, str] | None):
        self.igw = InternetGateway(
            name,
            self.vpc.id,
            tags,
            opts=ResourceOptions(parent=self, depends_on=[self.vpc]),
        )

    def __subnets(
        self,
        name: str,
        subnet_specs: List[NetworkSubnet],
        tags: Mapping[str, str] | None,
    ) -> None:
        self.subnets = {"private": [], "public": []}

        # ! Dirty code
        self.subnets_per_zone = {"private": {}, "public": {}}

        for i, subnet in enumerate(subnet_specs):
            settings = SubnetSettings(
                cidr_block=subnet.cidr_block,
                assign_ipv6_address_on_creation=subnet.assign_ipv6,
                availability_zone=subnet.availability_zone,
                map_public_ip_on_launch=subnet.map_public_ip,
            )

            subnet_name = f"{name}-{i}-{subnet.type}-subnet"
            _subnet = Subnet(
                subnet_name,
                self.vpc.id,
                tags,
                settings,
                opts=ResourceOptions(parent=self, depends_on=[self.vpc]),
            )
            self.subnets[subnet.type].append(_subnet)
            self.subnets_per_zone[subnet.type][subnet.availability_zone] = _subnet

    def __ngw(
        self, name: str, create_opt: str, tags: Mapping[str, str]
    ) -> Optional[Dict[str, ec2.NatGateway]]:
        self.ngws = []
        if create_opt == "per az":
            ngw_per_az = {}
            for az, subnet in self.subnets_per_zone["public"].items():
                ngw_name = f"{name}-{az}-ngw"
                ngw = NatGateway(
                    ngw_name,
                    subnet.id,
                    tags,
                    opts=ResourceOptions(parent=self, depends_on=[subnet]),
                )
                self.ngws.append(ngw)
                ngw_per_az[az] = ngw
            return ngw_per_az
        else:
            name = f"{name}-ngw"
            for az, subnet in self.subnets_per_zone["public"].items():
                subnet = subnet
                break
            self.ngws.append(
                NatGateway(
                    name,
                    subnet.id,
                    tags,
                    opts=ResourceOptions(parent=self, depends_on=[subnet]),
                )
            )
            return None

    def __route(
        self,
        name: str,
        igw_id: str,
        ngw_opts: str,
        private_subnets_ngw: Dict[str, List[ec2.Subnet | ec2.NatGateway]],
        tags: Mapping[str, str],
    ) -> None:
        self.route_tables = []

        # Public Route Table
        route_associations = []
        for i, subnet in enumerate(self.subnets["public"]):
            route_associations.append(RouteTableAssociations(f"public-{i}", subnet.id))
        name = f"{name}-route_table"
        route = Route("0.0.0.0/0", gateway_id=igw_id)

        depends_on = [self.vpc] + self.subnets["public"]
        self.route_tables.append(
            RouteTable(
                f"{name}-public",
                self.vpc.id,
                [route],
                route_associations,
                tags,
                ResourceOptions(parent=self, depends_on=depends_on),
            )
        )

        # Private Route Table
        if private_subnets_ngw and ngw_opts == "per az":
            for az, network in private_subnets_ngw.items():
                route_association = RouteTableAssociations(
                    f"private-{az}", network[0].id
                )

                route = Route("0.0.0.0/0", nat_gateway_id=network[1].id)
                self.route_tables.append(
                    RouteTable(
                        f"{name}-private-{az}",
                        self.vpc.id,
                        [route],
                        [route_association],
                        tags,
                        ResourceOptions(
                            parent=self, depends_on=[self.vpc, network[0], network[1]]
                        ),
                    )
                )
        elif ngw_opts == "single":
            for i, subnet in enumerate(self.subnets["private"]):
                route_association = RouteTableAssociations(f"private-{i}", subnet.id)

                route = Route("0.0.0.0/0", nat_gateway_id=self.ngws[0].id)

                self.route_tables.append(
                    RouteTable(
                        f"{name}-private-{i}",
                        self.vpc.id,
                        [route],
                        [route_association],
                        tags,
                        ResourceOptions(
                            parent=self, depends_on=[self.vpc, subnet, self.ngws[0]]
                        ),
                    )
                )
